#include "cmtlib.h"

/* ----------------------------------------------------------------
** Semaphores.
** ----------------------------------------------------------------
*/
/*man**************************************************************
NAME
	cmt_screate - create a semaphore

SYNOPSIS
	#include "cmtlib.h"

	CMT_SEM far * far cdecl cmt_screate(int initcount)

DESCRIPTION
	allocates space for semaphore and initializes it to the
	given value.

DIAGNOSTICS
	returns pointer to new semaphore or NULL.
******************************************************************/
CMT_SEM far * far cdecl cmt_screate(int initcount)
{
CMT_SEM far *n;
	n=cmt_malloc(sizeof(CMT_SEM));
	if (n) {
		n->count=initcount;
	}
	return n;
}

/*man**************************************************************
NAME
	cmt_sdelete - discard a semaphore

SYNOPSIS
	#include "cmtlib.h"

	CMT_SEM far * far cdecl cmt_sdelete(CMT_SEM far *sem)

DESCRIPTION
	discards a semaphore

DIAGNOSTICS
	always returns NULL
******************************************************************/
CMT_SEM far * far cdecl cmt_sdelete(CMT_SEM far *sem)
{
	if (sem)
		cmt_free(sem);
	return NULL;
}

/*man**************************************************************
NAME
	cmt_wait - wait on semaphore

SYNOPSIS
	#include "cmtlib.h"

	void far cdecl cmt_wait(CMT_SEM far *sem)

DESCRIPTION
	this function waits for a signal on semaphore. in our
	system this means decrementing semaphore value and
	then blocking until semaphore value goes back to 0.
******************************************************************/
void far cdecl cmt_wait(CMT_SEM far *sem)
{
	if (sem) {
		sem->count--;
		while (sem->count<0)
			cmt_pause();
	}
}

/*man**************************************************************
NAME
	cmt_signal - set a semaphore

SYNOPSIS
	#include "cmtlib.h"

	void far cdecl cmt_signal(CMT_SEM far *sem)

DESCRIPTION
	this function sets a 'signal' flag on semaphore. in our
	system we just incement semaphore value.
******************************************************************/
void far cdecl cmt_signal(CMT_SEM far *sem)
{
	if (sem)
		sem->count++;
}

